@@ -1,14 +1,13 @@
 from copy import deepcopy
-from dataclasses import dataclass
+from enum import Enum
 from functools import partial
-from pprint import pprint
 from typing import Callable, Self, Any
 
 from oldaplib.src.enums.action import Action
 from oldaplib.src.enums.attributeclass import AttributeClass
 from oldaplib.src.enums.haspropertyattr import HasPropertyAttr
-from oldaplib.src.enums.propertyclassattr import PropClassAttr
 from oldaplib.src.helpers.Notify import Notify
+from oldaplib.src.helpers.irincname import IriOrNCName
 from oldaplib.src.helpers.oldaperror import OldapErrorNotFound
 from oldaplib.src.helpers.serializer import serializer
 from oldaplib.src.iconnection import IConnection
@@ -18,10 +17,17 @@ from oldaplib.src.project import Project
 from oldaplib.src.propertyclass import PropertyClass, HasPropertyData
 from oldaplib.src.xsd.iri import Iri
 from oldaplib.src.xsd.xsd_datetime import Xsd_dateTime
-from oldaplib.src.xsd.xsd_decimal import Xsd_decimal
-from oldaplib.src.xsd.xsd_integer import Xsd_integer
 from oldaplib.src.xsd.xsd_ncname import Xsd_NCName
 from oldaplib.src.xsd.xsd_nonnegativeinteger import Xsd_nonNegativeInteger
+from oldaplib.src.xsd.xsd_qname import Xsd_QName
+
+
+@serializer
+class PropType(Enum):
+    INTERNAL = 1
+    STANDALONE = 2
+    EXTERNAL = 3
+
 
 @serializer
 class HasProperty(Model, Notify):
@@ -42,19 +48,24 @@ class HasProperty(Model, Notify):
                       as a single `HasPropertyData` object.
     :type haspropdata: HasPropertyData
     """
-    _prop: PropertyClass | Iri | None
+
+
+    _prop: PropertyClass | Xsd_QName | None
     _project: Project | None
+    _type: PropType | None
 
     def __init__(self, *,
                  con: IConnection,
-                 project: Project,
-                 prop: PropertyClass | Iri | None = None,
-                 notifier: Callable[[Iri], None] | None = None,
-                 notify_data: Iri | None = None,
+                 project: Project | Iri | Xsd_NCName | str,
+                 prop: PropertyClass | Xsd_QName | None = None,
+                 notifier: Callable[[Xsd_QName], None] | None = None,
+                 notify_data: Xsd_QName | None = None,
                  creator: Iri | None = None,  # DO NO USE! Only for jsonify!!
                  created: Xsd_dateTime | None = None,  # DO NO USE! Only for jsonify!!
                  contributor: Iri | None = None,  # DO NO USE! Only for jsonify!!
                  modified: Xsd_dateTime | None = None,  # DO NO USE! Only for jsonify!!
+                 validate: bool = False,
+                 _type: PropType | None = None,
                  **kwargs):
         """
         Initializes the class with the given parameters and configurations.
@@ -82,16 +93,28 @@ class HasProperty(Model, Notify):
                        contributor=contributor,
                        modified=modified)
         Notify.__init__(self, notifier, notify_data)
-        self._project = project
-        if isinstance(prop, Iri):
-            fixed_prop = Iri(str(prop).removesuffix("Shape"))
+        self._type = _type
+        if isinstance(project, Project):
+            self._project = project
+        else:
+            if not isinstance(project, (Iri, Xsd_NCName)):
+                project = IriOrNCName(project, validate=validate)
+            self._project = Project.read(self._con, project)
+        if isinstance(prop, Xsd_QName):
+            fixed_prop = Xsd_QName(str(prop).removesuffix("Shape"))
             try:
                 self._prop = PropertyClass.read(self._con, self._project, fixed_prop)
+                self._type = PropType.STANDALONE
             except OldapErrorNotFound as err:
                 self._prop = fixed_prop
+                self._type = PropType.EXTERNAL
         else:
             self._prop = prop
-        self._prop = prop
+            if self._type is None:
+                if prop.externalOntology:
+                    self._type = PropType.EXTERNAL
+                else:
+                    self._type = PropType.INTERNAL
         self.set_attributes(kwargs, HasPropertyAttr)
 
         for attr in HasPropertyAttr:
@@ -103,8 +126,8 @@ class HasProperty(Model, Notify):
         self._changeset = {}
 
     def update_notifier(self,
-                        notifier: Callable[[AttributeClass | Iri], None] | None = None,
-                        notify_data: HasPropertyAttr | Iri | None = None):
+                        notifier: Callable[[AttributeClass | Xsd_QName], None] | None = None,
+                        notify_data: HasPropertyAttr | Xsd_QName | None = None):
         self.set_notifier(notifier, notify_data)
         if isinstance(self._prop, PropertyClass):
             self._prop.set_notifier(self.notifier, self._prop.property_class_iri)
@@ -114,8 +137,9 @@ class HasProperty(Model, Notify):
 
     def _as_dict(self):
         return {x.fragment: y for x, y in self._attributes.items()} | super()._as_dict() | {
-            'project': self._project,
-            'prop': self._prop.property_class_iri if not self._prop.internal else self.prop
+            'project': self._project.projectShortName,
+            '_type': self._type,
+            'prop': self._prop
         }
 
     def __deepcopy__(self, memo: dict[Any, Any]) -> Self:
@@ -146,11 +170,15 @@ class HasProperty(Model, Notify):
         return s
 
     @property
-    def prop(self) -> PropertyClass | Iri | None:
+    def type(self) -> PropType:
+        return self._type
+
+    @property
+    def prop(self) -> PropertyClass | Xsd_QName | None:
         return self._prop
 
     @prop.setter
-    def prop(self, prop: PropertyClass | Iri) -> None:
+    def prop(self, prop: PropertyClass | Xsd_QName) -> None:
         self._prop = prop
 
     @property
@@ -160,12 +188,17 @@ class HasProperty(Model, Notify):
                                order=self._attributes.get(HasPropertyAttr.ORDER, None),
                                group=self._attributes.get(HasPropertyAttr.GROUP, None))
 
-    def notifier(self, attr: HasPropertyAttr | Iri) -> None:
+    def clear_changeset(self) -> None:
+        if hasattr(self._prop, 'clear_changeset'):
+            self._prop.clear_changeset()
+        super().clear_changeset()
+
+    def notifier(self, attr: HasPropertyAttr | Iri | Xsd_QName) -> None:
         #if attr == HasPropertyAttr.PROP:
         #    return
         if isinstance(attr, HasPropertyAttr):
             self._changeset[attr] = AttributeChange(self._attributes[attr], Action.MODIFY)
-        elif isinstance(attr, Iri):
+        elif isinstance(attr, Xsd_QName):
             self._changeset[attr] = AttributeChange(None, Action.MODIFY)
         self.notify()
 
@@ -182,7 +215,7 @@ class HasProperty(Model, Notify):
             sparql += f' ;\n{blank:{indent * indent_inc}}sh:group {self._attributes[HasPropertyAttr.GROUP].toRdf}'
         return sparql
 
-    def create_owl(self, indent: int = 0, indent_inc: int = 4):
+    def create_owl(self, indent: int = 0, indent_inc: int = 4) -> str:
         blank = ''
         sparql = ''
         min_count = Xsd_nonNegativeInteger(int(self._attributes[HasPropertyAttr.MIN_COUNT])) if self._attributes.get(HasPropertyAttr.MIN_COUNT) else None
@@ -199,13 +232,13 @@ class HasProperty(Model, Notify):
 
     def update_shacl(self,
                      graph: Xsd_NCName,
-                     resclass_iri: Iri,
-                     propclass_iri: Iri,
+                     resclass_iri: Xsd_QName,
+                     propclass_iri: Xsd_QName,
                      indent: int = 0, indent_inc: int = 4) -> str:
         blank = ''
         sparql_list = []
         for attr, change in self._changeset.items():
-            if isinstance(attr, Iri):  # if it's an IRI, the attached PropertyClass has changed. We don't process this here
+            if isinstance(attr, Xsd_QName):  # if it's an IRI, the attached PropertyClass has changed. We don't process this here
                 continue
             sparql = f'WITH {graph}:shacl\n'
             if change.action != Action.CREATE:
@@ -220,7 +253,7 @@ class HasProperty(Model, Notify):
 
             sparql += f'{blank:{indent * indent_inc}}WHERE {{\n'
             sparql += f'{blank:{(indent + 1) * indent_inc}}{resclass_iri}Shape sh:property ?prop .\n'
-            if isinstance(self._prop, Iri) or self._prop.internal is None:
+            if isinstance(self._prop, Xsd_QName) or self._prop.internal is None:
                 sparql += f'{blank:{(indent + 1) * indent_inc}}?prop sh:node {propclass_iri}Shape.\n'
             else:
                 sparql += f'{blank:{(indent + 1) * indent_inc}}?prop sh:path {propclass_iri} .\n'
@@ -233,8 +266,8 @@ class HasProperty(Model, Notify):
 
     def update_owl(self,
                      graph: Xsd_NCName,
-                     resclass_iri: Iri,
-                     propclass_iri: Iri,
+                     resclass_iri: Xsd_QName,
+                     propclass_iri: Xsd_QName,
                      indent: int = 0, indent_inc: int = 4) -> str:
         blank = ''
         if HasPropertyAttr.MIN_COUNT in self._changeset or HasPropertyAttr.MAX_COUNT in self._changeset:
@@ -271,3 +304,5 @@ class HasProperty(Model, Notify):
             sparql += f'{blank:{indent * indent_inc}}}}'
 
             return sparql
+        else:
+            return ''
