@@ -72,7 +72,7 @@ class ResourceInstance:
     :type changeset: dict[Iri, AttributeChange]
     """
     _iri: Iri
-    _values: dict[Iri, LangString | ObservableSet]
+    _values: dict[Xsd_QName, LangString | ObservableSet]
     _graph: Xsd_NCName
     _changeset: dict[Iri, AttributeChange]
 
@@ -99,12 +99,12 @@ class ResourceInstance:
         if iri and isinstance(iri, str):
             iri = Iri(Xsd_QName(self.project.projectShortName, iri))
         self._iri = Iri(iri, validate=True) if iri else Iri()
-        self._values = {}
+        self._values: dict[Xsd_QName, ValueType] = {}
         self._graph = self.project.projectShortName
         self._superclass_objs = {}
         self._changeset = {}
 
-        def set_values(propclass: dict[Iri, HasProperty]):
+        def set_values(propclass: dict[Xsd_QName, HasProperty]):
             for prop_iri, hasprop in propclass.items():
                 if kwargs.get(str(prop_iri)) or kwargs.get(prop_iri.fragment):
                     value = kwargs[str(prop_iri)] if kwargs.get(str(prop_iri)) else kwargs[prop_iri.fragment]
@@ -143,20 +143,22 @@ class ResourceInstance:
                         else:
                             self.validate_value(self._values[prop_iri], hasprop.prop)
 
-        def process_superclasses(superclass: dict[Iri, ResourceClass]):
-           for sc_iri, sc in superclass.items():
+        def process_superclasses(superclass: dict[Xsd_QName, ResourceClass]):
+            for sc_iri, sc in superclass.items():
+                if not sc:
+                    continue
                 if sc.superclass:
                     process_superclasses(sc.superclass)
-                if sc.owl_class_iri == Iri("oldap:Thing", validate=False):
+                if sc.owl_class_iri == Xsd_QName("oldap:Thing", validate=False):
                     timestamp = Xsd_dateTimeStamp()
-                    if not self._values.get(Iri('oldap:createdBy', validate=False)):
-                        self._values[Iri('oldap:createdBy', validate=False)] = ObservableSet({self._con.userIri})
-                    if not self._values.get(Iri('oldap:creationDate', validate=False)):
-                        self._values[Iri('oldap:creationDate', validate=False)] = ObservableSet({timestamp})
-                    if not self._values.get(Iri('oldap:lastModifiedBy', validate=False)):
-                        self._values[Iri('oldap:lastModifiedBy', validate=False)] = ObservableSet({self._con.userIri})
-                    if not self._values.get(Iri('oldap:lastModificationDate', validate=False)):
-                        self._values[Iri('oldap:lastModificationDate', validate=False)] = ObservableSet({timestamp})
+                    if not self._values.get(Xsd_QName('oldap:createdBy', validate=False)):
+                        self._values[Xsd_QName('oldap:createdBy', validate=False)] = ObservableSet({self._con.userIri})
+                    if not self._values.get(Xsd_QName('oldap:creationDate', validate=False)):
+                        self._values[Xsd_QName('oldap:creationDate', validate=False)] = ObservableSet({timestamp})
+                    if not self._values.get(Xsd_QName('oldap:lastModifiedBy', validate=False)):
+                        self._values[Xsd_QName('oldap:lastModifiedBy', validate=False)] = ObservableSet({self._con.userIri})
+                    if not self._values.get(Xsd_QName('oldap:lastModificationDate', validate=False)):
+                        self._values[Xsd_QName('oldap:lastModificationDate', validate=False)] = ObservableSet({timestamp})
                 set_values(sc.properties)
                 for iri, prop in sc.properties.items():
                     self.properties[iri] = prop
@@ -168,11 +170,33 @@ class ResourceInstance:
 
         for propname in self.properties.keys():
             setattr(ResourceInstance, propname.fragment, property(
-                partial(ResourceInstance.__get_value, prefix=propname.prefix, fragment=propname.fragment),
-                partial(ResourceInstance.__set_value, prefix=propname.prefix, fragment=propname.fragment),
-                partial(ResourceInstance.__del_value, prefix=propname.prefix, fragment=propname.fragment)))
+                partial(ResourceInstance.__get_value, attr=propname),
+                partial(ResourceInstance.__set_value, attr=propname),
+                partial(ResourceInstance.__del_value, attr=propname)))
         self.clear_changeset()
 
+    def __setitem__(self, key: Xsd_QName, value: ValueType | Xsd | None):
+        if self._values.get(key):
+            self.__set_value(value, key)
+        else:
+            raise OldapErrorValue(f'{self.name}: Property {key} does not exist.')
+
+    def __getitem__(self, key: Xsd_QName) -> ValueType | Xsd | None:
+        if self._values.get(key):
+            return self.__get_value(key)
+        else:
+            raise OldapErrorValue(f'{self.name}: Property {key} does not exist.')
+
+    def __delitem__(self, key: Xsd_QName):
+        if self._values.get(key):
+            self.__del_value(key)
+
+    def get(self, key: Xsd_QName) -> ValueType | Xsd | None:
+        if self._values.get(key):
+            return self._values[key]
+        else:
+            return None
+
     def validate_value(self, values: ValueType, property: PropertyClass):
         """
         Validates a set of values against a given property and its constraints. This function ensures
@@ -289,7 +313,7 @@ class ResourceInstance:
                     raise OldapErrorInconsistency(
                         f'Property {property} with LESS_THAN={property[PropClassAttr.LESS_THAN_OR_EQUALS]} has invalid value: "{max_value}" NOT LESS_THAN "{min_other_value}".')
 
-    def notifier(self, prop_iri: Iri):
+    def notifier(self, prop_iri: Xsd_QName):
         hasprop = self.properties[prop_iri]
         self.validate_value(self._values[prop_iri], hasprop)
 
@@ -328,16 +352,19 @@ class ResourceInstance:
             else:
                 return False, f'Actor does not have {permission} in project "{self.project.projectShortName}".'
 
-    def __get_value(self: Self, prefix: str, fragment: str) -> Xsd | ValueType | None:
-        attr = Iri(Xsd_QName(prefix, fragment, validate=False), validate=False)
+    def __get_value(self: Self, attr: Xsd_QName | str) -> Xsd | ValueType | None:
+        #attr = Xsd_QName(prefix, fragment, validate=False)
+        if not isinstance(attr, Xsd_QName):
+            attr = Xsd_QName(attr)
         tmp = self._values.get(attr, None)
         if tmp is not None and str(attr) in {'oldap:createdBy', 'oldap:creationDate', 'oldap:lastModifiedBy', 'oldap:lastModificationDate'}:
             return next(iter(tmp))
         return tmp
 
-    def __set_value(self: Self, value: ValueType | Xsd | None, prefix: str, fragment: str) -> None:
-        prop_iri = Iri(Xsd_QName(prefix, fragment, validate=False), validate=False)
-        hasprop = self.properties.get(prop_iri)
+    def __set_value(self: Self, value: ValueType | Xsd | None, attr: Xsd_QName | str) -> None:
+        if not isinstance(attr, Xsd_QName):
+            attr = Xsd_QName(attr)
+        hasprop = self.properties.get(attr)
 
         #
         # Validate
@@ -345,58 +372,58 @@ class ResourceInstance:
         if hasprop.get(HasPropertyAttr.MIN_COUNT):  # testing for MIN_COUNT conformance
             if hasprop[HasPropertyAttr.MIN_COUNT] > 0 and not value:
                 raise OldapErrorValue(
-                    f'{self.name}: Property {prop_iri} with MIN_COUNT={hasprop[HasPropertyAttr.MIN_COUNT]} is missing')
+                    f'{self.name}: Property {attr} with MIN_COUNT={hasprop[HasPropertyAttr.MIN_COUNT]} is missing')
             elif isinstance(value, (list, tuple, set, ObservableSet)) and len(value) < hasprop[HasPropertyAttr.MIN_COUNT]:
                 raise OldapErrorValue(
-                    f'{self.name}: Property {prop_iri} with MIN_COUNT={hasprop[HasPropertyAttr.MIN_COUNT]} has not enough values')
+                    f'{self.name}: Property {attr} with MIN_COUNT={hasprop[HasPropertyAttr.MIN_COUNT]} has not enough values')
         if hasprop.get(HasPropertyAttr.MAX_COUNT):  # testing for MAX_COUNT conformance
             if isinstance(value, (list, tuple, set, ObservableSet)) and len(value) > hasprop[HasPropertyAttr.MAX_COUNT]:
                 raise OldapErrorValue(
-                    f'{self.name}: Property {prop_iri} with MAX_COUNT={hasprop[HasPropertyAttr.MIN_COUNT]} has to many values (n={len(value)})')
+                    f'{self.name}: Property {attr} with MAX_COUNT={hasprop[HasPropertyAttr.MIN_COUNT]} has to many values (n={len(value)})')
         if value:
             if isinstance(value, LangString):
                 self.validate_value(value, hasprop.prop)
             else:
                 if isinstance(value, (list, tuple, set, ObservableSet)):
-                    for val in self._values[prop_iri]:
+                    for val in self._values[attr]:
                         self.validate_value(val, hasprop.prop)
                 else:
                     self.validate_value(value, hasprop.prop)
 
         if not value:
-            self._changeset[prop_iri] = AttributeChange(self._values.get(prop_iri), Action.DELETE)
-            del self._values[prop_iri]
+            self._changeset[attr] = AttributeChange(self._values.get(attr), Action.DELETE)
+            del self._values[attr]
         elif isinstance(value, (list, tuple, set, LangString)):  # we may have multiple values...
-            if self._values.get(prop_iri):
-                self._changeset[prop_iri] = AttributeChange(self._values.get(prop_iri), Action.REPLACE)
+            if self._values.get(attr):
+                self._changeset[attr] = AttributeChange(self._values.get(attr), Action.REPLACE)
             else:
-                self._changeset[prop_iri] = AttributeChange(None, Action.CREATE)
+                self._changeset[attr] = AttributeChange(None, Action.CREATE)
             if hasprop.prop.datatype == XsdDatatypes.langString:
-                self._values[prop_iri] = LangString(value, notifier=self.notifier, notify_data=prop_iri)
+                self._values[attr] = LangString(value, notifier=self.notifier, notify_data=attr)
             else:
-                self._values[prop_iri] = ObservableSet({
+                self._values[attr] = ObservableSet({
                     convert2datatype(x, hasprop.prop.datatype) for x in value
-                }, notifier=self.notifier, notify_data=prop_iri)
+                }, notifier=self.notifier, notify_data=attr)
         else:
-            if self._values.get(prop_iri):
-                self._changeset[prop_iri] = AttributeChange(self._values.get(prop_iri), Action.REPLACE)
+            if self._values.get(attr):
+                self._changeset[attr] = AttributeChange(self._values.get(attr), Action.REPLACE)
             else:
-                self._changeset[prop_iri] = AttributeChange(None, Action.CREATE)
+                self._changeset[attr] = AttributeChange(None, Action.CREATE)
             try:
-                self._values[prop_iri] = ObservableSet({convert2datatype(value, hasprop.prop.datatype)})
+                self._values[attr] = ObservableSet({convert2datatype(value, hasprop.prop.datatype)})
             except TypeError:
-                self._values[prop_iri] = convert2datatype(value, hasprop.prop.datatype)
+                self._values[attr] = convert2datatype(value, hasprop.prop.datatype)
 
-    def __del_value(self: Self, prefix: str, fragment: str) -> None:
-        prop_iri = Iri(Xsd_QName(prefix, fragment, validate=False), validate=False)
-        hasprop = self.properties.get(prop_iri)
+    def __del_value(self: Self, attr: Xsd_QName | str) -> None:
+        if not isinstance(attr, Xsd_QName):
+            attr = Xsd_QName(attr)
+        hasprop = self.properties.get(attr)
 
         if hasprop.get(HasPropertyAttr.MIN_COUNT):  # testing for MIN_COUNT conformance
             if hasprop[HasPropertyAttr.MIN_COUNT] > 0:
-                raise OldapErrorValue(f'{self.name}: Property {prop_iri} with MIN_COUNT={hasprop[HasPropertyAttr.MIN_COUNT]} cannot be deleted.')
+                raise OldapErrorValue(f'{self.name}: Property {attr} with MIN_COUNT={hasprop[HasPropertyAttr.MIN_COUNT]} cannot be deleted.')
 
-        self._changeset[prop_iri] = AttributeChange(self._values.get(prop_iri), Action.DELETE)
-        attr = Iri(Xsd_QName(prefix, fragment, validate=False), validate=False)
+        self._changeset[attr] = AttributeChange(self._values.get(attr), Action.DELETE)
         del self._values[attr]
 
     @property
@@ -404,7 +431,7 @@ class ResourceInstance:
         return self._iri
 
     @property
-    def changeset(self) -> dict[Iri, AttributeChange]:
+    def changeset(self) -> dict[Xsd_QName, AttributeChange]:
         return self._changeset
 
     def clear_changeset(self) -> None:
@@ -416,33 +443,24 @@ class ResourceInstance:
 
     def get_data_permission(self, context: Context, permission: DataPermission) -> bool:
         permission_query = context.sparql_context
-        # language=sparql
         permission_query += f'''
-        SELECT (COUNT(?permset) as ?numOfPermsets)
-        FROM oldap:onto
-        FROM shared:onto
-        FROM {self._graph}:onto
-        FROM NAMED oldap:admin
-        FROM NAMED {self._graph}:data
-        WHERE {{
-            BIND({self._iri.toRdf} as ?iri)
+        ASK {{
             GRAPH {self._graph}:data {{
-                ?iri oldap:grantsPermission ?permset .
+                {self._iri.toRdf} oldap:grantsPermission ?permset .
             }}
-            BIND({self._con.userIri.toRdf} as ?user)
             GRAPH oldap:admin {{
-                ?user oldap:hasPermissions ?permset .
+                {self._con.userIri.toRdf} oldap:hasPermissions ?permset .
                 ?permset oldap:givesPermission ?DataPermission .
                 ?DataPermission oldap:permissionValue ?permval .
             }}
             FILTER(?permval >= {permission.numeric.toRdf})
         }}'''
+
         if self._con.in_transaction():
-            jsonobj = self._con.transaction_query(permission_query)
+            result = self._con.transaction_query(permission_query)
         else:
-            jsonobj = self._con.query(permission_query)
-        res = QueryProcessor(context, jsonobj)
-        return res[0]['numOfPermsets'] > 0
+            result = self._con.query(permission_query)
+        return result['boolean']
 
     def create(self, indent: int = 0, indent_inc: int = 4) -> str:
         result, message = self.check_for_permissions(AdminPermission.ADMIN_CREATE)
@@ -451,10 +469,10 @@ class ResourceInstance:
 
         timestamp = Xsd_dateTimeStamp()
         if self.name == "Thing":
-            self._values[Iri('oldap:createdBy', validate=False)] = ObservableSet({self._con.userIri})
-            self._values[Iri('oldap:creationDate', validate=False)] = ObservableSet({timestamp})
-            self._values[Iri('oldap:lastModifiedBy', validate=False)] = ObservableSet({self._con.userIri})
-            self._values[Iri('oldap:lastModificationDate', validate=False)] = ObservableSet({timestamp})
+            self._values[Xsd_QName('oldap:createdBy', validate=False)] = ObservableSet({self._con.userIri})
+            self._values[Xsd_QName('oldap:creationDate', validate=False)] = ObservableSet({timestamp})
+            self._values[Xsd_QName('oldap:lastModifiedBy', validate=False)] = ObservableSet({self._con.userIri})
+            self._values[Xsd_QName('oldap:lastModificationDate', validate=False)] = ObservableSet({timestamp})
 
         indent: int = 0
         indent_inc: int = 4
@@ -505,26 +523,19 @@ class ResourceInstance:
         context = Context(name=con.context_name)
         sparql = context.sparql_context
         sparql += f'''
-SELECT ?predicate ?value
-FROM oldap:onto
-FROM shared:onto
-FROM {graph}:onto
-FROM NAMED oldap:admin
-FROM NAMED {graph}:data
-WHERE {{
-	BIND({iri.toRdf} as ?iri)
-    GRAPH {graph}:data {{
-        ?iri ?predicate ?value .
-        ?iri oldap:grantsPermission ?permset .
-    }}
-    BIND({con.userIri.toRdf} as ?user)
-    GRAPH oldap:admin {{
-    	?user oldap:hasPermissions ?permset .
-    	?permset oldap:givesPermission ?DataPermission .
-    	?DataPermission oldap:permissionValue ?permval .
-    }}
-    FILTER(?permval >= {DataPermission.DATA_VIEW.numeric.toRdf})
-}}'''
+        SELECT ?predicate ?value
+        WHERE {{
+            GRAPH {graph}:data {{
+                {iri.toRdf} ?predicate ?value .
+                {iri.toRdf} oldap:grantsPermission ?permset .
+            }}
+            GRAPH oldap:admin {{
+                {con.userIri.toRdf} oldap:hasPermissions ?permset .
+                ?permset oldap:givesPermission ?DataPermission .
+                ?DataPermission oldap:permissionValue ?permval .
+            }}
+            FILTER(?permval >= {DataPermission.DATA_VIEW.numeric.toRdf})
+        }}'''
         jsonres = con.query(sparql)
         res = QueryProcessor(context, jsonres)
         objtype = None
@@ -687,12 +698,13 @@ WHERE {{
 
         context = Context(name=self._con.context_name)
         inuse = context.sparql_context
-        inuse = f'''
-        SELECT (COUNT(?res) as ?nres)
-        WHERE {{
-            ?res ?prop {self._iri.toRdf} .
+        inuse += f"""
+        ASK {{
+            GRAPH ?g {{
+                ?res ?prop {self._iri.toRdf} .
+            }}
         }}
-        '''
+        """
 
         context = Context(name=self._con.context_name)
         sparql = context.sparql_context
@@ -710,9 +722,8 @@ WHERE {{
                 self._con.transaction_abort()
                 raise OldapErrorNoPermission(f'No permission to update resource "{self._iri}"')
         try:
-            jsonobj = self._con.transaction_query(inuse)
-            res = QueryProcessor(context, jsonobj)
-            if res[0]['nres'] > 0:
+            result = self._con.transaction_query(inuse)
+            if result['boolean']:
                 raise OldapErrorInUse(f'Resource "{self._iri}" is in use and cannot be deleted.')
         except OldapError:
             self._con.transaction_abort()
@@ -769,10 +780,7 @@ class ResourceInstanceFactory:
         else:
             self._project = Project.read(self._con, project)
 
-        self._datamodel = DataModel.read(con=self._con, project=self._project)
-
-        #self._oldap_project = Project.read(self._con, "oldap")
-        #self._oldap_datamodel = DataModel.read(con=self._con, project=self._oldap_project)
+        self._datamodel = DataModel.read(con=self._con, project=self._project, ignore_cache=True)
 
     def createObjectInstance(self, name: Xsd_NCName | str) -> Type:  ## ToDo: Get name automatically from IRI
         classiri = Xsd_QName(self._project.projectShortName, name)
@@ -789,6 +797,46 @@ class ResourceInstanceFactory:
             'properties': resclass.properties,
             'superclass': resclass.superclass})
 
+    @staticmethod
+    def read_data(con: IConnection, projectShortName: Xsd_NCName | str, iri: Iri | str) -> dict[Xsd_QName, Any]:
+        if not isinstance(iri, Iri):
+            iri = Iri(iri, validate=True)
+        if not isinstance(projectShortName, Xsd_NCName):
+            graph = Xsd_NCName(projectShortName)
+        else:
+            graph = projectShortName
+
+        context = Context(name=con.context_name)
+        sparql = context.sparql_context
+        sparql += f'''
+        SELECT ?predicate ?value
+        WHERE {{
+            GRAPH {graph}:data {{
+                {iri.toRdf} ?predicate ?value .
+                {iri.toRdf} oldap:grantsPermission ?permset .
+            }}
+            GRAPH oldap:admin {{
+            	{con.userIri.toRdf} oldap:hasPermissions ?permset .
+            	?permset oldap:givesPermission ?DataPermission .
+            	?DataPermission oldap:permissionValue ?permval .
+            }}
+            FILTER(?permval >= {DataPermission.DATA_VIEW.numeric.toRdf})
+        }}
+        '''
+        jsonres = con.query(sparql)
+        res = QueryProcessor(context, jsonres)
+        data = {}
+        for r in res:
+            if r['predicate'].is_qname:
+                if not data.get(r['predicate'].as_qname):
+                    data[r['predicate'].as_qname] = []
+                data[str(r['predicate'].as_qname)].append(str(r['value']))
+            else:
+                raise OldapErrorInconsistency(f"Expected QName as predicate, got {r['predicate']}")
+        if not data.get('rdf:type'):
+            raise OldapErrorNotFound(f'Resource with iri <{iri}> not found.')
+        return data
+
     def read(self, iri: Iri | str) -> ResourceInstance:
         if not isinstance(iri, Iri):
             iri = Iri(iri, validate=True)
@@ -797,11 +845,6 @@ class ResourceInstanceFactory:
         sparql = context.sparql_context
         sparql += f'''
         SELECT ?predicate ?value
-        FROM oldap:onto
-        FROM shared:onto
-        FROM {graph}:onto
-        FROM NAMED oldap:admin
-        FROM NAMED {graph}:data
         WHERE {{
         	BIND({iri.toRdf} as ?iri)
             GRAPH {graph}:data {{
@@ -845,20 +888,24 @@ class ResourceInstanceFactory:
         Instance = self.createObjectInstance(objtype)
         return Instance(iri=iri, **kwargs)
 
-    def search_fulltext(self, s: str, count_only: bool = False, limit: int = 100, offset: int = 0) -> int | dict[Iri, dict[str, Xsd]]:
-        graph = self._project.projectShortName
-        context = Context(name=self._con.context_name)
+    @staticmethod
+    def search_fulltext(con: IConnection,
+                        projectShortName: Xsd_NCName | str,
+                        s: str,
+                        count_only: bool = False,
+                        limit: int = 100,
+                        offset: int = 0) -> int | dict[Iri, dict[str, Xsd]]:
+        if not isinstance(projectShortName, Xsd_NCName):
+            graph = Xsd_NCName(projectShortName)
+        else:
+            graph = projectShortName
+        context = Context(name=con.context_name)
         sparql = context.sparql_context
         if (count_only):
             sparql += "SELECT (COUNT(DISTINCT ?s) as ?numResult)"
         else:
             sparql += "SELECT DISTINCT ?s ?t ?p ?o"
         sparql += f'''
-        FROM oldap:onto
-        FROM shared:onto
-        FROM {graph}:onto
-        FROM NAMED oldap:admin
-        FROM NAMED {graph}:data
         WHERE {{
             GRAPH {graph}:data {{
                 ?s ?p ?o .
@@ -868,9 +915,8 @@ class ResourceInstanceFactory:
             FILTER(isLiteral(?o) && 
                 (datatype(?o) = xsd:string || datatype(?o) = rdf:langString || lang(?o) != ""))
             FILTER(CONTAINS(LCASE(STR(?o)), "{s}"))  # case-insensitive substring match
-            BIND({self._con.userIri.toRdf} as ?user)
             GRAPH oldap:admin {{
-    	        ?user oldap:hasPermissions ?permset .
+    	        {con.userIri.toRdf} oldap:hasPermissions ?permset .
     	        ?permset oldap:givesPermission ?DataPermission .
     	        ?DataPermission oldap:permissionValue ?permval .
             }}
@@ -879,21 +925,31 @@ class ResourceInstanceFactory:
         '''
         if not count_only:
             sparql += f'LIMIT {limit} OFFSET {offset}'
-        jsonres = self._con.query(sparql)
+        try:
+            jsonres = con.query(sparql)
+        except OldapError:
+            print(sparql)
+            raise
         res = QueryProcessor(context, jsonres)
         if count_only:
-            if isinstance(res[0]['numResult'], Xsd_integer):
-                tmp = cast(Xsd_integer, res[0]['numResult'])
-                return int(tmp)
-            else:
-                raise OldapErrorInconsistency(f'Expected integer as value, got "{res[0]["numResult"]}"')
+            return res[0]['numResult']
         else:
-            result: dict[Iri, dict[str, Xsd]] = {}
-            for r in res:
-                iri = cast(Iri, r['s'])
-                resclass = cast(Iri, r['t'])
-                result[iri] = {'resclass': resclass, 'property': r['p'], 'value': r['o']}
-            return result
+            return {Iri(r['s']): {r['p']: r['o']} for r in res}
+        # jsonres = con.query(sparql)
+        # res = QueryProcessor(context, jsonres)
+        # if count_only:
+        #     if isinstance(res[0]['numResult'], Xsd_integer):
+        #         tmp = cast(Xsd_integer, res[0]['numResult'])
+        #         return int(tmp)
+        #     else:
+        #         raise OldapErrorInconsistency(f'Expected integer as value, got "{res[0]["numResult"]}"')
+        # else:
+        #     result: dict[Iri, dict[str, Xsd]] = {}
+        #     for r in res:
+        #         iri = cast(Iri, r['s'])
+        #         resclass = cast(Iri, r['t'])
+        #         result[iri] = {'resclass': resclass, 'property': r['p'], 'value': r['o']}
+        #     return result
 
 
 
