@@ -2,8 +2,9 @@ import json
 from copy import deepcopy
 from dataclasses import dataclass
 from functools import partial
+from pprint import pprint
 
-from typing import List, Self, Any
+from typing import List, Self, Any, Callable
 from datetime import date, datetime
 
 from oldaplib.src.cachesingleton import CacheSingletonRedis
@@ -13,6 +14,7 @@ from oldaplib.src.helpers.context import Context
 from oldaplib.src.enums.action import Action
 from oldaplib.src.dtypes.namespaceiri import NamespaceIRI
 from oldaplib.src.helpers.irincname import IriOrNCName
+from oldaplib.src.helpers.observable_dict import ObservableDict
 from oldaplib.src.helpers.serializer import serializer
 from oldaplib.src.helpers.tools import lprint
 from oldaplib.src.xsd.iri import Iri
@@ -231,6 +233,8 @@ class Project(Model):
         """
         self._changeset[attr] = AttributeChange(self._attributes[attr], Action.MODIFY)
 
+
+
     @classmethod
     def read(cls,
              con: IConnection,
@@ -295,15 +299,17 @@ class Project(Model):
                     tmp._con = con
                     return tmp
             query += f"""
-                SELECT ?proj ?prop ?val
-                FROM NAMED oldap:admin
+                SELECT ?proj ?prop ?val ?prefix ?iri
                 WHERE {{
                     GRAPH oldap:admin {{
                         ?proj a oldap:Project .
-                        ?proj oldap:projectShortName ?shortname .
+                        ?proj oldap:projectShortName {shortname.toRdf} .
                         ?proj ?prop ?val .
+                        OPTIONAL {{
+                            ?val oldap:prefix ?prefix .
+                            ?val oldap:fullIri ?iri
+                        }}
                     }}
-                    FILTER(?shortname = {shortname.toRdf})
                 }}
             """
         jsonobj = con.query(query)
@@ -345,10 +351,10 @@ class Project(Model):
                 case 'oldap:projectEnd':
                     projectEnd = r['val']
         if label:
-            label.changeset_clear()
+            label.clear_changeset()
             label.set_notifier(cls.notifier, Xsd_QName(ProjectAttr.LABEL.value))
         if comment:
-            comment.changeset_clear()
+            comment.clear_changeset()
             comment.set_notifier(cls.notifier, Xsd_QName(ProjectAttr.COMMENT.value))
         context[projectShortName] = namespaceIri
         instance = cls(con=con,
@@ -458,6 +464,17 @@ class Project(Model):
 
         context = Context(name=self._con.context_name)
 
+        sparql0 = context.sparql_context
+        sparql0 += f"""
+        ASK {{
+            GRAPH oldap:admin {{
+    	        ?p a oldap:Project ;
+    		        oldap:projectShortName ?sname .
+    	        FILTER(?sname = {self.projectShortName.toRdf})
+	        }}
+        }}
+        """
+
         sparql1 = context.sparql_context
         sparql1 += f"""
         SELECT ?project
@@ -467,6 +484,23 @@ class Project(Model):
             FILTER(?project = {self.projectIri.toRdf})
         }}
         """
+        sparql1a = context.sparql_context
+        sparql1a += f"""
+        ASK {{
+            GRAPH oldap:admin {{
+                ?project oldap:namespaceIri {self.namespaceIri.toRdf} .
+            }}
+        }}
+        """
+
+        sparql1b = context.sparql_context
+        sparql1b += f"""
+        ASK {{
+            GRAPH oldap:admin {{
+                {self.projectIri.toRdf} ?p ?o .
+            }}
+        }}
+        """
 
         blank = ''
         sparql2 = context.sparql_context
@@ -485,11 +519,20 @@ class Project(Model):
         sparql2 += f'{blank:{indent * indent_inc}}}}\n'
 
         self._con.transaction_start()
-        jsonobj = self.safe_query(sparql1)
-        res = QueryProcessor(context, jsonobj)
-        if len(res) > 0:
+        result = self.safe_query(sparql0)
+        if result['boolean']:
             self._con.transaction_abort()
-            raise OldapErrorAlreadyExists(f'A Project with a projectIri "{self.projectIri}" already exists')
+            raise OldapErrorAlreadyExists(f'A Project with a shortname "{self.projectShortName}" already exists.')
+
+        result = self.safe_query(sparql1a)
+        if result['boolean']:
+            self._con.transaction_abort()
+            raise OldapErrorAlreadyExists(f'A Project with a namespace IRI "{self.namespaceIri}" already exists.')
+
+        result = self.safe_query(sparql1b)
+        if result['boolean']:
+            self._con.transaction_abort()
+            raise OldapErrorAlreadyExists(f'A Project with a project IRI "{self.projectIri}" already exists.')
 
         self.safe_update(sparql2)
         self.safe_commit()
@@ -520,6 +563,22 @@ class Project(Model):
         :raises OldapErrorUpdateFailed: Raised if the update fails due to timestamp mismatch or other inconsistencies.
         :raises OldapError: Raised for other internal errors.
         """
+
+        def dict_diff(a: dict, b: dict) -> dict:
+            """
+            Compare two dicts and return added, removed, and changed key–values.
+            """
+            a_keys = set(a) if a else set()
+            b_keys = set(b) if b else set()
+            shared = a_keys & b_keys
+
+            return {
+                'added': {k: b[k] for k in b_keys - a_keys},
+                'removed': {k: a[k] for k in a_keys - b_keys},
+                'changed': {k: (a[k], b[k]) for k in shared if a[k] != b[k]},
+                'same': {k: a[k] for k in shared if a[k] == b[k]},
+            }
+
         result, message = self.check_for_permissions()
         if not result:
             raise OldapErrorNoPermission(message)
@@ -545,20 +604,26 @@ class Project(Model):
                                                             field=Xsd_QName(field.value))
                     sparql_list.append(sparql)
                 continue
+
             sparql = f'{blank:{indent * indent_inc}}# Project field "{field.value}" with action "{change.action.value}"\n'
-            sparql += f'{blank:{indent * indent_inc}}WITH oldap:admin\n'
             if change.action != Action.CREATE:
                 sparql += f'{blank:{indent * indent_inc}}DELETE {{\n'
-                sparql += f'{blank:{(indent + 1) * indent_inc}}?project {field.value} {change.old_value.toRdf} .\n'
+                sparql += f'{blank:{(indent + 1) * indent_inc}}GRAPH oldap:admin {{\n'
+                sparql += f'{blank:{(indent + 2) * indent_inc}}?project {field.value} {change.old_value.toRdf} .\n'
+                sparql += f'{blank:{(indent + 1)* indent_inc}}}}\n'
                 sparql += f'{blank:{indent * indent_inc}}}}\n'
             if change.action != Action.DELETE:
                 sparql += f'{blank:{indent * indent_inc}}INSERT {{\n'
-                sparql += f'{blank:{(indent + 1) * indent_inc}}?project {field.value} {self._attributes[field].toRdf} .\n'
+                sparql += f'{blank:{(indent + 1) * indent_inc}}GRAPH oldap:admin {{\n'
+                sparql += f'{blank:{(indent + 2) * indent_inc}}?project {field.value} {self._attributes[field].toRdf} .\n'
+                sparql += f'{blank:{(indent + 1) * indent_inc}}}}\n'
                 sparql += f'{blank:{indent * indent_inc}}}}\n'
             sparql += f'{blank:{indent * indent_inc}}WHERE {{\n'
             sparql += f'{blank:{(indent + 1) * indent_inc}}BIND({self.projectIri.toRdf} as ?project)\n'
             if change.action != Action.CREATE:
-                sparql += f'{blank:{(indent + 1) * indent_inc}}?project {field.value} {change.old_value.toRdf} .\n'
+                sparql += f'{blank:{(indent + 1) * indent_inc}}GRAPH oldap:admin {{\n'
+                sparql += f'{blank:{(indent + 2) * indent_inc}}?project {field.value} {change.old_value.toRdf} .\n'
+                sparql += f'{blank:{(indent + 1) * indent_inc}}}}\n'
             sparql += f'{blank:{indent * indent_inc}}}}'
             sparql_list.append(sparql)
         sparql = context.sparql_context
@@ -570,6 +635,7 @@ class Project(Model):
             self.set_modified_by_iri(Xsd_QName('oldap:admin'), self.projectIri, self.modified, timestamp)
             modtime = self.get_modified_by_iri(Xsd_QName('oldap:admin'), self.projectIri)
         except OldapError:
+            print(sparql)
             self._con.transaction_abort()
             raise
         if timestamp != modtime:
