@@ -6,6 +6,8 @@ from pathlib import Path
 from pprint import pprint
 from time import sleep
 
+from oldaplib.src.cachesingleton import CacheSingletonRedis
+from oldaplib.src.datamodel import DataModel
 from oldaplib.src.objectfactory import ResourceInstanceFactory
 from oldaplib.src.connection import Connection
 from oldaplib.src.enums.action import Action
@@ -106,7 +108,7 @@ class TestObjectFactory(unittest.TestCase):
         sleep(1)  # upload may take a while...
 
         user = User.read(cls._connection, "rosenth")
-        user.hasPermissions.add(Iri('oldap:GenericUpdate'))
+        user.hasPermissions.add(Iri('oldap:GenericUpdate'))  # TODO: SHOULD WORK WITH Xsd_QName
         user.update()
 
         ps = PermissionSet(con=cls._connection,
@@ -129,6 +131,8 @@ class TestObjectFactory(unittest.TestCase):
                     isActive=True)
         user.create()
         cls._tuser = User.read(cls._connection, "factorytestuser")
+        cache = CacheSingletonRedis()
+        cache.clear()
 
 
     @classmethod
@@ -337,7 +341,28 @@ class TestObjectFactory(unittest.TestCase):
                  grantsPermission=Iri('oldap:GenericView'))
         b.create()
 
+    def test_constructor_D(self):
+        dm = DataModel.read(con=self._connection, project='test')
+        factory = ResourceInstanceFactory(con=self._connection, project='test')
+        Cat = factory.createObjectInstance('Cat')
+
+        mycat = Cat(name="Fluffy", subName="Fluffy the Cat")
+        mycat.create()
+
     def test_read_A(self):
+        factory = ResourceInstanceFactory(con=self._connection, project='test')
+        Book = factory.createObjectInstance('Book')
+        b = Book(title="The Life and Times of Scrooge",
+                 author="test:TuomasHolopainen",
+                 pubDate="2001-01-01",
+                 grantsPermission=Iri('oldap:GenericView'))
+        b.create()
+        data = ResourceInstanceFactory.read_data(con=self._connection, iri=b.iri, projectShortName='test')
+        self.assertEqual(data['rdf:type'], ['test:Book'])
+        self.assertEqual(data['test:title'], ['The Life and Times of Scrooge'])
+        self.assertEqual(data['test:author'], ['test:TuomasHolopainen'])
+
+    def test_read_B(self):
         factory = ResourceInstanceFactory(con=self._connection, project='test')
         Book = factory.createObjectInstance('Book')
         b = Book(title="The Life and Times of Scrooge",
@@ -354,6 +379,24 @@ class TestObjectFactory(unittest.TestCase):
         self.assertEqual(jsonobj['test:pubDate'], ['2001-01-01'])
         self.assertEqual(jsonobj['test:title'], ['The Life and Times of Scrooge'])
 
+    def test_read_C(self):
+        factory = ResourceInstanceFactory(con=self._connection, project='test')
+        Book = factory.createObjectInstance('Book')
+        b = Book(title="The Life and Times of Scrooge",
+                 author="test:TuomasHolopainen",
+                 pubDate="2001-01-01",
+                 grantsPermission=Iri('oldap:GenericView'))
+        b.create()
+        bb = Book.read(con=self._connection, iri=b.iri)
+        self.assertEqual(bb.name, "Book")
+        self.assertEqual(bb.title, {Xsd_string("The Life and Times of Scrooge")})
+        self.assertEqual(bb.author, {Iri("test:TuomasHolopainen")})
+        jsonobj = bb.toJsonObject()
+        self.assertEqual(jsonobj['test:author'], ['test:TuomasHolopainen'])
+        self.assertEqual(jsonobj['test:pubDate'], ['2001-01-01'])
+        self.assertEqual(jsonobj['test:title'], ['The Life and Times of Scrooge'])
+
+
     def test_value_setter(self):
         factory = ResourceInstanceFactory(con=self._connection, project='test')
         SetterTester = factory.createObjectInstance('SetterTester')
@@ -407,7 +450,7 @@ class TestObjectFactory(unittest.TestCase):
         self.assertEqual(obj2.decimalSetter, {Xsd_decimal(3.14159), Xsd_decimal(2.71828), Xsd_decimal(1.61803)})
         self.assertFalse(obj2.integerSetter)
 
-    def test_value_modifier(self):
+    def test_value_modifier_A(self):
         factory = ResourceInstanceFactory(con=self._connection, project='test')
         SetterTester = factory.createObjectInstance('SetterTester')
         obj1 = SetterTester(stringSetter="This is a test string",
@@ -433,6 +476,32 @@ class TestObjectFactory(unittest.TestCase):
         self.assertEqual(obj1.integerSetter, {Xsd_int(20), Xsd_int(42)})
         self.assertFalse(obj1.booleanSetter)
 
+    def test_value_modifier_B(self):
+        factory = ResourceInstanceFactory(con=self._connection, project='test')
+        SetterTester = factory.createObjectInstance('SetterTester')
+        obj1 = SetterTester(stringSetter="This is a test string",
+                            langStringSetter=LangString("C'est un teste@fr", "Dies ist eine Test-Zeichenkette@de"),
+                            decimalSetter=Xsd_decimal(3.14),
+                            integerSetter={-10, 20},
+                            booleanSetter=True,
+                            grantsPermission={Iri('oldap:GenericView'), Iri('oldap:GenericUpdate')})
+        obj1.create()
+        obj1 = SetterTester.read(con=self._connection, iri=obj1.iri)
+        obj1[Xsd_QName('test:langStringSetter')][Language.FR] = "Qu'est-ce que c'est?"
+        obj1[Xsd_QName('test:integerSetter')].add(42)
+        obj1[Xsd_QName('test:integerSetter')].discard(-10)
+        obj1[Xsd_QName('test:booleanSetter')] = False
+        with self.assertRaises(OldapErrorValue):
+            obj1[Xsd_QName('test:stringSetter')].pop()
+        with self.assertRaises(OldapErrorValue):
+            del obj1[Xsd_QName('test:stringSetter')]
+        obj1.update()
+        obj1 = SetterTester.read(con=self._connection, iri=obj1.iri)
+        self.assertEqual(obj1.stringSetter, {"This is a test string"})
+        self.assertEqual(obj1.langStringSetter, LangString("Dies ist eine Test-Zeichenkette@de", "Qu'est-ce que c'est?@fr"))
+        self.assertEqual(obj1.integerSetter, {Xsd_int(20), Xsd_int(42)})
+        self.assertFalse(obj1.booleanSetter)
+
     def test_value_maxcount_mincount(self):
         factory = ResourceInstanceFactory(con=self._connection, project='test')
         Person = factory.createObjectInstance('Person')
@@ -576,7 +645,9 @@ class TestObjectFactory(unittest.TestCase):
                      pubDate="1995-09-27",
                      grantsPermission=Iri('oldap:GenericView'))
             b.create()
-        res = factory.search_fulltext('tales')
+        res = ResourceInstanceFactory.search_fulltext(con=self._connection,
+                                                      projectShortName='test',
+                                                      s='tales')
 
         for x, y in res.items():
             print(x, y)
